/*

Cellular Automata Demo
======================

This program displays the results of a 1D binary celluar automata structure
as described in Steven Wolfram's _A New Kind of Science_.

Each pixel represents a single cell.  Each cell can take on the value 0 (white)
or 1 (black).  Each consecutive row of pixels represents the next step in the evolution
of the cellular structure.

Growth occurs as follows: The state of a cell at stage N+1 is determined by one of 8 rules
that is selected based on the state of the cell and both of its neighbors in stage N.  For the
purpose of computing border cell colors, cells outside the screen are considered white.

Example:

The rule set {1,1,1 -> 0; 1,1,0 -> 0; 1,0,1 -> 0; 1,0,0 -> 1; 0,1,1 -> 1; 0,1,0 -> 1; 0,0,1->1, 0,0,0->0}
defines that a cell with left neighbor value 1, self value 1, and right neighbor value 0 will take on the
value 0 in the next step.  Wolfram calls this "Rule 30" since the values of the three-state sets will equal
30 if you interpret each state to be a binary digit, the sum of the state sets in which the result is 1 will be
30.

Expected behavior:

When this program runs, a triangular region of the screen will evidence signs of cellular behavior.
The leftmost edge will show signs of regularity throught the entire structure but the rightmost region
will show no obvious signs of repetition and is in fact statistically random.

*/

#include <nuon/bios.h>
#include <nuon/dma.h>
#include <nuon/joystick.h>
#include <nuon/video.h>
#include <nuon/mml2d.h>
#include <string.h>

//32 bit color values taken from mml2d.h

#define NTSCWHITE (0xB4808000UL)
#define WHITE (0xEB808000UL)
#define BLACK (0x10808000UL)
#define FIRST_ROW (60UL)
#define LAST_ROW (419UL)
#define NUM_COLORS (2UL)

//Use 32 bit 8-8-8-Alpha full-screen mode
#define SCREEN_WIDTH (720UL)
#define SCREEN_HEIGHT (480UL)

#define MAIN_CHANNEL_WIDTH SCREEN_WIDTH
#define MAIN_CHANNEL_HEIGHT SCREEN_HEIGHT
#define ROW_RENDER_OFFSET MAIN_CHANNEL_HEIGHT

#define ROW_DATA_LENGTH (MAIN_CHANNEL_WIDTH + (MAIN_CHANNEL_HEIGHT << 1UL))

unsigned long rowData[2][ROW_DATA_LENGTH];
unsigned long ruleColors[8];
unsigned long *localScratch;
unsigned long colors[] = { WHITE, BLACK };
unsigned long *mainChannelBuffer;
int localSize;

VidDisplay mainDisplay;
VidChannel mainChannel;

extern unsigned long splash[];

void DisplaySplashScreen();
void ClearScreen(unsigned long color);

void vidsetup(void)
{

  mainChannel.dmaflags = ((MAIN_CHANNEL_WIDTH >> 3) << 16) | ePixDma | eCluster | (4UL<<4);
  mainChannelBuffer =
    (unsigned long *)_MemAlloc((MAIN_CHANNEL_WIDTH * MAIN_CHANNEL_HEIGHT) << 2, 512, kMemSDRAM);

  _VidSetup((void *)mainChannelBuffer,mainChannel.dmaflags, MAIN_CHANNEL_WIDTH, MAIN_CHANNEL_HEIGHT, 0);
}

void RenderRow(unsigned long *ptrRowData, unsigned long y)
{
  unsigned long x = 0;

  for(x = 0; x < MAIN_CHANNEL_WIDTH; x += 72)
  {    
    _DMALinear((72UL<<16) | (1UL<<13), (void *)(&ptrRowData[x]), localScratch);
    _DMABiLinear(mainChannel.dmaflags, mainChannelBuffer, (72UL<<16)|x, (1UL<<16)|y, (void *)localScratch);
  }
}

void RenderFirstRow(unsigned long currRowArrayIndex, unsigned long y)
{
  unsigned long x;

  for(x = 0; x < ROW_DATA_LENGTH; x++)
  {
    rowData[currRowArrayIndex][x] = WHITE;
  }

  x = ROW_DATA_LENGTH >> 1;
  rowData[currRowArrayIndex][x] = BLACK;

  RenderRow(&rowData[currRowArrayIndex][ROW_RENDER_OFFSET],y);
}

unsigned long GetColorValue(unsigned long color)
{
  if(color == colors[1])
  {
    return 0x01UL;
  }
  
  return 0x00UL;
}

void SetRuleColors(unsigned long ruleNumber)
{
  //Assign colors to each subrule 0 through 7 based on the
  //binary representation of the specified rule number
  //e.g. ruleNumber 30 results in {0,0,0,1,1,1,1,0}

  unsigned long i;

  for(i = 0UL; i < 8UL; i++)
  {
    ruleColors[i] = ruleNumber & 0x01UL;
    ruleNumber >>= 1;
  }
}

unsigned long GetRuleColorIndex(unsigned long i,unsigned long *ptr)
{
  //Calculate the subrule to use based on the value of the cell and its neighbors in the previous step
  //Cells offscreen are assumed to have a value of 0 (white)

  unsigned long lval, rval,tempval;

  (i > 0) ? (lval = GetColorValue(*(ptr - 1))) : (lval = 0UL);
  (i < (ROW_DATA_LENGTH - 1)) ? (rval = GetColorValue(*(ptr+1))) : (rval = 0UL);

  return (lval << 2) | (GetColorValue(*ptr) << 1) | rval;
}

void ClearScreen(unsigned long color)
{
  unsigned long x,y;

  for(y = 0; y < MAIN_CHANNEL_HEIGHT; y = y + 8)
  {
    for (x = 0; x < MAIN_CHANNEL_WIDTH; x = x + 8)
    {
      _DMABiLinear(mainChannel.dmaflags|DMA_DIRECT_BIT, mainChannelBuffer, (8<<16)|x, (8<<16)|y, (void *)color);
    }
  }
}

void DisplaySplashScreen(void)
{
  unsigned long x,y;
  unsigned long *ptrScanline;

  ptrScanline = splash;

  for(y = 0; y < MAIN_CHANNEL_HEIGHT; y++)
  {
    for(x = 0; x < MAIN_CHANNEL_WIDTH; x += 72)
    {    
      _DMALinear((72UL<<16) | (1UL<<13), (void *)(&ptrScanline[x]), localScratch);
      _DMABiLinear(mainChannel.dmaflags, mainChannelBuffer, (72UL<<16)|x, (1UL<<16)|y, (void *)localScratch);
    }
    
    ptrScanline += MAIN_CHANNEL_WIDTH;
  }  
}

int main(void)
{
  unsigned long *ptrCurrRowData,*ptrPrevRowData;
  unsigned long x,y,currentRowArrayIndex;
  unsigned long buttonStatus;
  bool bJoystickActive;
  unsigned long ruleNumber = 30;


  localScratch = (unsigned long *)_MemLocalScratch(&localSize);

  vidsetup();

  //Detect joysticks
  _DeviceDetect(1);

  if(_Controller[1].status)
  {
    bJoystickActive = true;
  }
  else
  {
    bJoystickActive = false;
  }

  DisplaySplashScreen();
  while(!(ButtonA(_Controller[1])) && bJoystickActive)
  {
  }

  while(1)
  {
    ClearScreen(kWhite);
    SetRuleColors(ruleNumber);

    //Render the first row of pixels
    currentRowArrayIndex = 0;

    y = FIRST_ROW;
    RenderFirstRow(currentRowArrayIndex,y);

    currentRowArrayIndex = 1;

    //Render each row of pixels (cells) based on the rule set

    for(y = FIRST_ROW + 1; y < LAST_ROW; y++)
    {
      //At first pixel of row
      ptrPrevRowData = &rowData[1 - currentRowArrayIndex][0];
      ptrCurrRowData = &rowData[currentRowArrayIndex][0];

      for(x = 0; x < ROW_DATA_LENGTH; x++)
      {
        //Get color of pixel based on ruleset
        ptrCurrRowData[x] = colors[ruleColors[GetRuleColorIndex(x,&ptrPrevRowData[x])]];
      }

      //Render the current row
      RenderRow(&ptrCurrRowData[ROW_RENDER_OFFSET],y);

      //Set currentRowArrayIndex to the non-current index (0->1, 1->0)
      currentRowArrayIndex = 1 - currentRowArrayIndex;

      // Now at first pixel of next row
    }

    ruleNumber = (ruleNumber + 1) & 0xFFUL;

    while(!(ButtonA(_Controller[1])) && bJoystickActive)
    {
    }

  }

  _MemFree(mainChannelBuffer);
  y = 0xDEADBEEF;

  return 1;
}
